package be.ac.info.fundp.TVLParser.SyntaxTree;

import be.ac.info.fundp.TVLParser.exceptions.AmbiguousReferenceException;
import be.ac.info.fundp.TVLParser.exceptions.ChildrenFeatureNotFoundException;
import be.ac.info.fundp.TVLParser.exceptions.IllegalExpressionException;
import be.ac.info.fundp.TVLParser.exceptions.SharedFeatureUsingParentSelectorException;
import be.ac.info.fundp.TVLParser.exceptions.SymbolNotFoundException;
import be.ac.info.fundp.TVLParser.symbolTables.FeaturesSymbolTable;

public class ExcludesExpression implements BooleanExpression {
	
	String longID1, longID2;
	String normalFormLongID1;
	String normalFormLongID2;
	
	FeaturesSymbolTable featuresSymbolTable;
	
	public ExcludesExpression(String longID1, String longID2, FeaturesSymbolTable featuresSymbolTable) {
		this.longID1 = longID1;
		this.longID2 = longID2;
		this.featuresSymbolTable = featuresSymbolTable;
	}

	/**
	 * @return the featureID1
	 */
	public String getLongID1() {
		return this.longID1;
	}

	/**
	 * @return the featureID2
	 */
	public String getLongID2() {
		return this.longID2;
	}

	/**
	 * @return the featureIDPath1
	 */
	
	public String toString() {
		return this.longID1+" excludes "+this.longID2.toString();
	}

	@Override
	public int getType() throws AmbiguousReferenceException, IllegalExpressionException, SymbolNotFoundException, ChildrenFeatureNotFoundException, SharedFeatureUsingParentSelectorException {
		if (featuresSymbolTable.containsSymbol(longID1)) {
			this.normalFormLongID1 = (String) featuresSymbolTable.getSymbol(longID1).get(1);
			if (featuresSymbolTable.containsSymbol(longID2)) {
				this.normalFormLongID2 = (String) featuresSymbolTable.getSymbol(longID2).get(1);
				return Expression.BOOL;
			}
			else {
				throw new IllegalExpressionException("Type error : the expression "+this.toString()+" is invalid. The right paramater ( "+longID2+" ) is not a valid feature ID. In an excludes expression, each parameter must be a valid feature ID.");
			}
		}
		else {
			throw new IllegalExpressionException("Type error : the expression "+this.toString()+" is invalid. The let paramater ( "+longID1+" ) is not a valid feature ID. In an excludes expression, each parameter must be a valid feature ID.");
		}
	}
	
	public Expression toNormalForm() {
		return new ExcludesExpression(this.normalFormLongID1, this.normalFormLongID2, null);
	}
	
	public BooleanExpression toSimplifiedForm() {
		return new OrExpression(new NotExpression(new LongIDExpression(this.longID1, this.featuresSymbolTable)), new NotExpression(new LongIDExpression(this.longID2, this.featuresSymbolTable)));
	}
	
	/** This method is normally never used because the toSimplifiedForm() method removes each XorAggExpression from the diagram.
	 * 
	 * @return
	 */
	public BooleanExpression removeArrows() {
		return this;
	}
	
	/** This method is normally never used because the toSimplifiedForm() method removes each XorAggExpression from the diagram.
	 * 
	 * @return
	 */
	public BooleanExpression distributeDisjunctions() {
		return this;
	}

	/** This method is normally never used because the toSimplifiedForm() method removes each XorAggExpression from the diagram.
	 * 
	 * @return
	 */
	public BooleanExpression distributeNegations() {
		return this;
	}
	
}
