package be.ac.info.fundp.TVLParser.SyntaxTree;

import be.ac.info.fundp.TVLParser.Util.Util;
import be.ac.info.fundp.TVLParser.exceptions.AmbiguousReferenceException;
import be.ac.info.fundp.TVLParser.exceptions.BadTypeUseException;
import be.ac.info.fundp.TVLParser.exceptions.ChildrenFeatureNotFoundException;
import be.ac.info.fundp.TVLParser.exceptions.ExpressionOutOfBoundException;
import be.ac.info.fundp.TVLParser.exceptions.ExpressionTypeViolatingAttributeTypeException;
import be.ac.info.fundp.TVLParser.exceptions.IDEnumValuesConflictException;
import be.ac.info.fundp.TVLParser.exceptions.IllegalExpressionException;
import be.ac.info.fundp.TVLParser.exceptions.SetExpressionMemberOutOfBoundException;
import be.ac.info.fundp.TVLParser.exceptions.SetExpressionMemberViolatingAttributeTypeException;
import be.ac.info.fundp.TVLParser.exceptions.SharedFeatureUsingParentConstructorException;
import be.ac.info.fundp.TVLParser.exceptions.SharedFeatureUsingParentSelectorException;
import be.ac.info.fundp.TVLParser.exceptions.SymbolNotFoundException;

public class ParenthesesExpression implements BooleanExpression {

	public Expression expression;
	
	public ParenthesesExpression(Expression expression) {
		this.expression = Util.removeParentheses(expression);
	}
	
	public String toString() {
		return "( "+this.expression.toString()+" )";
	}
	
	public int getType() throws IllegalExpressionException, SymbolNotFoundException, AmbiguousReferenceException, SharedFeatureUsingParentConstructorException, BadTypeUseException, SharedFeatureUsingParentSelectorException, ChildrenFeatureNotFoundException, IDEnumValuesConflictException, NumberFormatException, SetExpressionMemberOutOfBoundException, SetExpressionMemberViolatingAttributeTypeException, ExpressionTypeViolatingAttributeTypeException, ExpressionOutOfBoundException {
		return this.expression.getType();
	}
	
	public Expression toNormalForm() {
		return new ParenthesesExpression(this.expression.toNormalForm());
	}

	@Override
	public BooleanExpression removeArrows() {
		return new ParenthesesExpression(((BooleanExpression)this.expression).removeArrows());
	}

	@Override
	public BooleanExpression toSimplifiedForm() {
		return new ParenthesesExpression(((BooleanExpression) this.expression).toSimplifiedForm());
	}
	
	public Expression getExpression() {
		return this.expression;
	}

	@Override
	public BooleanExpression distributeNegations() {
		return new ParenthesesExpression(((BooleanExpression)this.expression).distributeNegations());
	}
	
	public BooleanExpression distributeDisjunctions() {
		return ((BooleanExpression) this.expression).distributeDisjunctions();
	}	
}
